#! /home/song3/anaconda3/bin/python

# This code is partially adapted from https://github.com/jgibbard/iqtool/blob/master/iqgen.py

import numpy as np
from matplotlib import pyplot as plt
import sys, struct
import os.path
import warnings
warnings.simplefilter('ignore')

from bitio import bit_open
from bitarray import bitarray


def complexToSingleArray(array):
    #Convert compex array to real array when 
    #I and Q are stored one after each other
    #This is done because numpy can only write 
    #complex numbers as to a file in certain data types     

    realArray = np.real(array)
    imagArray = np.imag(array)

    #Create array double length of input to hold both real and imag values
    output = np.zeros(realArray.size + imagArray.size)

    #Pack  real (I) and imag (Q) values in the correct order
    #By default numpy writes index 0 to an output file first
    output[0::2] = realArray
    output[1::2] = imagArray

    return output


def readFile(filename):
    ret = []
    with bit_open(filename, "r") as f:
         while True:
            try:
                num = f.read_bits(2)
            except:
                break
            ret.append(num)

    return ret



def intToComplex(num):
    output = []
    modulation = np.array([1+1j,-1+1j,-1-1j,1-1j])
    for value in num:
        output.append(modulation[value])

    output = np.array(output)
    return output

def addNoise(signal):
    np.random.seed(1234)
    real_noise = np.random.normal(0,0.25,len(signal))
    imag_noise = np.random.normal(0,0.3,len(signal))
    noise = real_noise +1j*imag_noise
    signal += noise 
    return signal


def writeFile(filename,signal):
    # data = complexToSingleArray(signal)
    data = signal.astype(np.complex64)
    data.tofile(filename)
    

if __name__ == "__main__":
    numbers = readFile("pic1.jpg")
    signal = intToComplex(numbers)
    signal = addNoise(signal)
    writeFile("modulated_pic.dat",signal)



